import os
import sys
import time
import threading

import colorama
import paramiko

colorama.init(autoreset=True)


def create_ssh_connection(config: dict):
    """create new ssh connection"""
    cmd = f"sshpass -p {config['pass']} ssh {config['user']}@{config['host']} -o StrictHostKeyChecking=no"
    if config.get("port"):
        cmd += f" -p {config['port']}"
    os.system(cmd)


def execute_by_ssh_connection(config: dict, command: str):
    client = paramiko.SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    client.connect(config['host'], port=config['port'], username=config['user'], password=config['pass'], compress=True)
    stdin, stdout, stderr = client.exec_command(command)

    def stdout_bind():
        for each in stdout:
            sys.stdout.write(each)

    def stderr_bind():
        for each in stderr:
            sys.stderr.write(each)

    stdout_thread = threading.Thread(target=stdout_bind, daemon=True)
    stderr_thread = threading.Thread(target=stderr_bind, daemon=True)

    stdout_thread.start()
    stderr_thread.start()

    stdout_thread.join()
    stderr_thread.join()

    client.close()



import os
import json

import colorama


class FileConfig:

    def __init__(self, path: str):
        self.__path = path
        self.__data = []
        self.load()
        assert isinstance(self.data, list)

    @property
    def data(self):
        return self.__data

    @property
    def path(self):
        return self.__path

    def load(self):
        try:
            with open(self.path, "r") as file:
                self.__data = json.load(file)
        except FileNotFoundError:
            pass

    def save(self):
        with open(self.path, "w+") as file:
            json.dump(self.data, file, indent=4)

    def update(self, item: dict):
        assert item.get("user") is not None
        assert item.get("host") is not None
        assert item.get("pass") is not None
        for each in self.data:
            if item['user'] == each['user'] and item['host'] == each['host']:
                each.update(item)
                break
        else:
            self.__data.append(item)
        self.save()

    def delete(self, index: int):
        del self.data[index]
        self.save()


def create_conf():
    """create and return FileConfig object"""
    if os.name == "posix":
        root = "~"
    else:
        root = os.environ.get("windir", "C:")
    config = FileConfig(os.path.join(root, 'messh.conf'))
    return config



import os
import sys
import json
import getpass
import argparse

import colorama
import click

from .config import FileConfig, create_conf
from .ssh import create_ssh_connection, execute_by_ssh_connection


config = create_conf()


@click.group()
def main():
    pass


def show_list():
    print("Configuration list")
    if config.data:
        for i, item in enumerate(config.data):
            print("- ", end="")
            print(f"[{i}] {colorama.Fore.GREEN}{item['user']}@{item['host']:<16}", end=" ")
            print(f"{item['name']}")
    else:
        print("  ", end="")
        print(colorama.Fore.BLUE+"Empty list")
    print()


@main.command(name="list", help="display all ssh configurations")
def display():
    show_list()


@main.command(help="delete a ssh configuration")
@click.argument('index', type=int)
def delete(index):
    config.delete(index)


@main.command(help="edit nickname for a ssh configuration")
@click.argument('index', type=int)
def edit(index):
    item = config.data[index]
    click.secho("input new name for", nl=False)
    click.secho(f" {item['user']}@{item['host']}", nl=False, fg='green')
    click.secho(" : ", nl=False)
    name = input()
    if name:  # if not a emtpy string
        item['name'] = name
    config.update(item)


@main.command(help="connect target host by ssh")
@click.argument('target')
@click.option('--port', metavar='port', type=int, help='port, an integer, default 22', default=22)
@click.option('--name', metavar='name', help="host nickname will display", default="")
@click.option('--only-create', ' /-O', metavar='only-create', help="only update a ssh config without connect", default=False)
def connect(target, port, name, only_create):
    try:
        target = int(target)
        item = config.data[target]
    except ValueError:
        try:
            target = target.split("@")
            assert len(target) == 2
            password = getpass.getpass()
            item = {
                'user': target[0],
                'host': target[1],
                'port': port,
                'name': name,
                'pass': password,
            }
            config.update(item)
        except AssertionError:
            click.secho("Must input target like: user@host or index of config", fg='red')
            sys.exit(1)
    except IndexError:
        click.secho("Over index!", fg='red', nl=False)
        show_list()
        sys.exit(1)
    # create ssh connection
    if not only_create:
        create_ssh_connection(item)


@main.command(help="execute command by ssh")
@click.argument('target')
@click.argument('command')
@click.option('--port', metavar='port', type=int, help='port, an integer, default 22', default=22)
def execute(target, command, port):
    try:
        target = int(target)
        item = config.data[target]
    except ValueError:
        try:
            target = target.split("@")
            assert len(target) == 2
            password = getpass.getpass()
            item = {
                'user': target[0],
                'host': target[1],
                'port': port,
                'pass': password,
            }
        except AssertionError:
            click.secho("Must input target like: user@host or index of config", fg='red')
            sys.exit(1)
    except IndexError:
        click.secho("Over index!", fg='red', nl=False)
        show_list()
        sys.exit(1)
    click.secho("Execute command: ", nl=False)
    click.secho(command, fg='blue')
    execute_by_ssh_connection(item, command)


if __name__ == "__main__":
    main()
